﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace RoboRaceAuto
{
  public enum TypPolicka
  {
    PasNahoru = 0,
    PasDoleva = 1,
    PasDolu = 2,
    PasDoprava = 3,
    Volno,
    Zed,
    Dira,
    Poklad
  }
  public enum TypPohybu
  {
    Zpet,
    Vpred1 = 1,
    Vpred2 = 2,
    Vpred3 = 3,
    Doleva,
    Doprava
  }

  public class Lerak
  {
    public static int[] Pohyby = new int[] { 0, -1, 1, 0, 0, 1, -1, 0 };

    public int X;
    public int Y;
    public int Smer;
    public RoboRace Race;

    public Lerak Clone()
    {
      return new Lerak(this.Race, this.X, this.Y, this.Smer);
    }
    
    public Lerak(RoboRace race, int x, int y, int smer)
    {
      this.Race = race;
      this.X = x;
      this.Y = y;
      this.Smer = smer;
    }

    public Lerak(RoboRace race, int x, int y, char smer)
    {
      this.Race = race;
      this.X = x;
      this.Y = y;
      switch(smer)
      {
        case 'N': this.Smer = 0; break;
        case 'R': this.Smer = 1; break;
        case 'D': this.Smer = 2; break;
        case 'L': this.Smer = 3; break;
      }
    }

    protected bool Posun(int smer, out bool dira)
    {
      int tempX = this.X + Lerak.Pohyby[smer * 2];
      int tempY = this.Y + Lerak.Pohyby[smer * 2 + 1];
      dira = false;
      if(tempX < 0 || tempY < 0 || tempX > 19 || tempY > 19 || Race.Mapa[tempX, tempY] == TypPolicka.Dira)
      {
        this.X = tempX;
        this.Y = tempY;
        dira = true;
        return false;
      }
      else if(Race.Mapa[tempX, tempY] == TypPolicka.Zed)
      {
        return false;
      }
      this.X = tempX;
      this.Y = tempY;
      return true;
    }

    public bool Pohyb(TypPohybu pohyb)
    {
      bool dira;
      switch(pohyb)
      {
        case TypPohybu.Doleva:
          this.Smer = (this.Smer + 3) % 4;
          break;
        case TypPohybu.Doprava:
          this.Smer = (this.Smer + 1) % 4;
          break;
        case TypPohybu.Zpet:
          if(!this.Posun((this.Smer + 2) % 4, out dira) && dira)
            return false;
          break;
        default:
          for(int i = 0; i < (int)pohyb; i++)
          {
            if(!this.Posun(this.Smer, out dira))
            {
              if(dira)
                return false;
              else
                i = 4;
            }
          }
          break;
      }
      if(this.X < 0 || this.Y < 0 || this.X > 19 || this.Y > 19)
        return false;

      if((int)Race.Mapa[this.X, this.Y] < 4)
        if(!this.Posun((int)Race.Mapa[this.X, this.Y], out dira) && dira)
          return false;

      if(this.X < 0 || this.Y < 0 || this.X > 19 || this.Y > 19)
        return false;
      return true;
    }

    public bool Souhlasi(Lerak druhy)
    {
      return this.X == druhy.X && this.Y == druhy.Y && this.Smer == druhy.Smer;
    }
  }

  public class RoboRace
  {
    public TypPolicka[,] Mapa;

    public static TypPohybu ZnakNaTypPohybu(char znak)
    {
      switch(znak)
      {
        default:
        case '1': return TypPohybu.Vpred1;
        case '2': return TypPohybu.Vpred2;
        case '3': return TypPohybu.Vpred3;
        case 'L': return TypPohybu.Doleva;
        case 'R': return TypPohybu.Doprava;
        case 'Z': return TypPohybu.Zpet;
      }
    }
    public static char TypPohybuNaZnak(TypPohybu typ)
    {
      switch(typ)
      {
        default:
        case TypPohybu.Vpred1: return '1';
        case TypPohybu.Vpred2: return '2';
        case TypPohybu.Vpred3: return '3';
        case TypPohybu.Doleva: return 'L';
        case TypPohybu.Doprava: return 'R';
        case TypPohybu.Zpet: return 'Z';
      }
    }
    public static TypPolicka ZnakNaTypPolicka(char znak)
    {
      switch(znak)
      {
        default:
        case '.': return TypPolicka.Volno;
        case 'X': return TypPolicka.Zed;
        case '#': return TypPolicka.Dira;
        case '$': return TypPolicka.Poklad;
        case '^': return TypPolicka.PasNahoru;
        case 'v': return TypPolicka.PasDolu;
        case '<': return TypPolicka.PasDoprava;
        case '>': return TypPolicka.PasDoleva;
      }
    }

    public RoboRace(string[] mapa, int offset)
    {
      this.Mapa = new TypPolicka[20, 20];

      for(int iY = 0; iY < 20; iY++)
        for(int iX = 0; iX < 20; iX++)
          this.Mapa[iX, iY] = RoboRace.ZnakNaTypPolicka(mapa[iY + offset][iX]);
    }
  }
}
